/******************************************************************************
 a controller for a 128K async SRAM. 
 Makes it work like a BRAM
******************************************************************************/
module mSRAM128K(
  input sclk
, input en              //1 means do enable;
, input we
, input  [16: 0] ab
, input  [ 7: 0] di
, output reg [ 7: 0] do
// interface to the real chip
, output [16: 0] xab
, inout  [ 7: 0] xdb
, output  xcs_
, output  xoe_
, output  xwe_
);
  assign xcs_ = ~en;    //chip select is inverted
  assign xwe_ = ~we;    //we is inverted
  assign xoe_ = 0;     //output on (low) when reading
  assign xab  = ab;     //address bus permanently connected
  //
  // Implement a tri-state circuit for the inout xdb
  // controls the chip for this cycle.
  // Reading a deselected chip will return 0 if pulldowns are enabled.
  wire [7:0] xin = we ? 8'bZ : xdb[7:0] ; //on read, let data in    
  assign xdb[7:0]     = we ? di[7:0]   : 8'bZ; //on write, let data out  
  //
  // Return read result next cycle
  always @ (posedge sclk)
    if(en & ~we)  //on write, output 0 - for some reason it's FF
      do[7:0] <= xin[7:0];
    else
      do[7:0] <= 8'h00;
          

endmodule
