/******************************************************************************
 UART  - wrapper around the Xilinx Picoblaze uart.
 
 Files required:
   bbfifo_16x8.v
   kcuart_rx.v
   kcuart_tx.v
   uart_rx.v
   uart-tx.v

To connect to arlet's 6502 core, I decode an 8-byte memory location (with SEL).
Base address is the status register when read.  Base+1 is the data register,
which can be read or written.  In the future I may add other registers.
      
******************************************************************************/
module mUART#(
  parameter WID=16
)(
  input CLK
, input [2:0] addr
, input SEL
, input WE
, input [7:0] DI
, output reg [7:0] DO
// interface to hardware
, output TX
, input RX
, output URDY_ //TODO: rename this properly when bugs are fixed
);
//-----------------------------------------------------------------------------
// The UART requires pulses at the rate of BAUD * 16.  At 30MHz, use a 16-bit
// shift register with a single bit on...This provides 115200 * 16 (close to)
//SRL16 #(.INIT(1)) baudshifter1 (baud_x16,1,1,1,1,CLK,baud_x16);
reg [WID:0] shr = 1;
always @ (posedge CLK) begin
  shr[WID:1] <= shr[WID-1:0];
  shr[0] <= shr[WID];
end
wire baud_x16 = shr[0];

//
wire tx_full;
wire tx_half_full;
wire rx_full;
wire rx_half_full;
wire rx_data_present;
wire [7:0] rx_data;
wire do_tx;
wire do_rx;
//-----------------------------------------------------------------------------
// uart tx
//-----------------------------------------------------------------------------
uart_tx utx(
    DI,                     //data to send out
    do_tx,
    0,                      //reset buffer
    baud_x16,               // baud rate x 16 pulses
    TX,                     // output pin
    tx_full,
    tx_half_full,
    CLK);
//-----------------------------------------------------------------------------
// uart rx
//-----------------------------------------------------------------------------
uart_rx urx(
    RX,                 //input pin
    rx_data[7:0],       //input data
    do_rx,              //when strobed, consider read done
    0,                  //rx reset buffer
    baud_x16,           //baud 
    rx_data_present,
    rx_full,
    rx_half_full,
    CLK);  
//-----------------------------------------------------------------------------
// mux for the output value: even=status odd=data 
always @ (posedge CLK) 
  if(SEL & ~WE)
    if(addr[0]) 
        DO <= rx_data[7:0];
    else
      DO <= {1'b0,rx_data_present, rx_full, rx_half_full,2'b00, tx_full, tx_half_full};
  else
    DO <= 8'h00;
  
assign do_tx = WE & SEL;
assign do_rx = ~WE & SEL & addr[0];  //any odd address acks read

assign URDY_ = rx_half_full;             //ready when input buffer not full!
endmodule
