/******************************************************************************
 A 6502 system

 To change speed:  60MHz clock is divided by DIV and multiplied by MUL in VDCM
 also, remember to change the UART divider:
 uartdiv = FREQ / 16 / 115200 (rounded).  Set terminal for 2 stop bits...
 round down - a faster uart clock seems to work better.\
.002  LED IO is not bit0 of C000
******************************************************************************/

 module top(

  input   clk
, output  led

, input URX, output UTX
, input RESET
// SRAM 
, output SRAM_CS_, output SRAM_WE_, output SRAM_OE_
, output [15:0] SRAM_A,  inout  [7:0] SRAM_D

, output [7:0] PORTA
, input [7:0] PORTB
);
/*assign SRAM_CS_ = 1;
assign SRAM_WE_ = 1;
assign SRAM_OE_ = 1;
assign SRAM_A = 0;
assign SRAM_D = 0;
*/

// Memory address selects
wire BRAM_SEL;
wire UART_SEL;
wire IO_SEL;  //C0xx
wire LED_SEL;
wire PORTA_SEL;
wire PORTB_SEL;
wire SRAM_SEL;

wire CPU_CLK   ;
VDCM  #(.DIV(12),.MUL(9)) myvdcm(
  .CLKIN_IN(clk)
, .RST_IN(0)
, .CLKFX_OUT(CPU_CLK));

// Handle reset pin synchronization
reg CPU_RESET,rst1; //for bringing reset into the clock domain
always @ (posedge CPU_CLK) begin
  rst1 <= RESET;
  CPU_RESET  <= rst1;
end

wire [15:0] CPU_AB;
wor [7:0] CPU_DI;
wire [7:0] CPU_DO;
wire CPU_WE;

cpu mycpu(
  .clk(CPU_CLK),
  .reset(CPU_RESET),
  .AB(CPU_AB[15:0]),
  .DI(CPU_DI[7:0]),
  .DO(CPU_DO[7:0]),
  .WE(CPU_WE),
  .IRQ(0),
  .NMI(0),
  .RDY(1)

); 





assign IO_SEL = (CPU_AB[15:8]==8'hC0); 

// Memory address selects
//E000-FFFF is BRAM
// Note: calling these BRAM_HI and BRAM_LO crashes Xilinx tools!
wire BRAM_L = (CPU_AB[15:10] ==6'b000000); //low 4 pages
wire BRAM_H = (CPU_AB[15:10] ==6'b111111); //high 4 pages
assign BRAM_SEL = (BRAM_H | BRAM_L);

//assign BRAM_SEL = (CPU_AB[15:11]==5'b11111)|(CPU_AB[15:9]==7'b0000000);
//assign BRAM_SEL = (CPU_AB[15:8]==8'hFF)|(CPU_AB[15:11]==5'h00);
assign LED_SEL =      IO_SEL & (CPU_AB[7:3]==5'h00000); // LED         is at C000
assign UART_SEL  =    IO_SEL & (CPU_AB[7:3]==5'b00001); // UART        is at C008
assign PORTA_SEL =    IO_SEL & (CPU_AB[7:3]==5'b00010); // output port is at C010
assign PORTB_SEL =    IO_SEL & (CPU_AB[7:3]==5'b00011); // input  port is at C018
//assign UART_SEL =    IO_SEL & (CPU_AB[7:3]==5'h1); 
assign SRAM_SEL = ~(IO_SEL | BRAM_SEL);

wire [7:0] UART_DO;
//wire URX_CLEAN;
//SRL16 shifter(URX_CLEAN,1,1,0,0,CPU_CLK,URX);
wire URDY_;
//  30MHz 16
//  35MHz 18
//  40    21
//  45    24
//  50    27
//  55    29
//  60    32
//  70    37.9 
//  80    43.4
//  90    48.82
// 100    54.25
// 105    56.96
mUART #(.WID(24)) uart(
  CPU_CLK
, CPU_AB[2:0]
, UART_SEL
, CPU_WE
, CPU_DO[7:0]     //cpu-->uart
, CPU_DI
, UTX
, URX
, URDY_
);

wire [7:0] BRAM_DO;
BRAM2K  sysbram(CPU_CLK,{BRAM_H,CPU_AB[9:0]},CPU_DI,CPU_DO[7:0],BRAM_SEL,CPU_WE);

wire [7:0] SRAM_DO;
mSRAM128K sram(
  .sclk(CPU_CLK)
, .en(SRAM_SEL)
, .we(CPU_WE)
, .ab(CPU_AB)
, .di(CPU_DO[7:0])
, .do(CPU_DI)
, .xab(SRAM_A)
, .xdb(SRAM_D[7:0])  
, .xcs_(SRAM_CS_)
, .xoe_(SRAM_OE_)
, .xwe_(SRAM_WE_)
);

// Parallel port A and B
reg [7:0]PORTA_REG;
always @(posedge CPU_CLK)
  if(PORTA_SEL & CPU_WE)
    PORTA_REG[7:0] <= CPU_DO[7:0];

assign PORTA[7:0] = PORTA_REG[7:0];

reg [7:0] PORTB_1;
reg [7:0] PORTB_2;
reg [7:0] PORTB_DO;
always @(posedge CPU_CLK) begin 
  PORTB_1 <= PORTB;
  PORTB_2 <= PORTB_1;
  if(PORTB_SEL & ~CPU_WE)
   PORTB_DO <= PORTB_2;
  else
   PORTB_DO <= 0;
end
assign CPU_DI = PORTB_DO;


// LED at C000 - Aug 27,2013: bit 0 
reg rled=1;
always @(posedge CPU_CLK)
  if(LED_SEL & CPU_WE)
    rled <= ~CPU_DO[0];//~rled;
assign led = rled ;

endmodule
